#
# Copyright (c) 2022, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
__all__ = ["Float"]

import typing

from neptune.attributes.atoms.copiable_atom import CopiableAtom
from neptune.common.warnings import (
    NeptuneUnsupportedValue,
    warn_once,
)
from neptune.internal.container_type import ContainerType
from neptune.internal.operation import AssignFloat
from neptune.internal.types.utils import is_unsupported_float
from neptune.types.atoms.float import Float as FloatVal

if typing.TYPE_CHECKING:
    from neptune.internal.backends.neptune_backend import NeptuneBackend


class Float(CopiableAtom):
    @staticmethod
    def create_assignment_operation(path, value: float):
        return AssignFloat(path, value)

    @staticmethod
    def getter(
        backend: "NeptuneBackend",
        container_id: str,
        container_type: ContainerType,
        path: typing.List[str],
    ) -> float:
        val = backend.get_float_attribute(container_id, container_type, path)
        return val.value

    def assign(self, value: typing.Union[FloatVal, float, int], *, wait: bool = False):
        if not isinstance(value, FloatVal):
            value = FloatVal(value)

        if is_unsupported_float(value.value):
            warn_once(
                message=f"WARNING: The value you're trying to log is a nonstandard float value ({str(value.value)}) "
                f"that is not currently supported. "
                f"We'll add support for this type of value in the future. "
                f"For now, you can use utils.stringify_unsupported() to log one or more values as strings: "
                f"run['field'] = stringify_unsupported(float({str(value.value)}))",
                exception=NeptuneUnsupportedValue,
            )
            return

        with self._container.lock():
            self._enqueue_operation(self.create_assignment_operation(self._path, value.value), wait=wait)



#
# Copyright (c) 2024, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

__all__ = ["ANONYMOUS_API_TOKEN_CONTENT", "DO_NOT_TRACK_GIT_REPOSITORY"]

ANONYMOUS_API_TOKEN_CONTENT = (
    "eyJhcGlfYWRkcmVzcyI6Imh0dHBzOi8vYXBwLm5lcHR1bmUuYWkiLCJhcGlfdXJsIjoiaHR0cHM6Ly9hcHAubmVwdHVuZS"
    "5haSIsImFwaV9rZXkiOiJiNzA2YmM4Zi03NmY5LTRjMmUtOTM5ZC00YmEwMzZmOTMyZTQifQo="
)
DO_NOT_TRACK_GIT_REPOSITORY = "DO_NOT_TRACK_GIT_REPOSITORY"



#
# Copyright (c) 2022, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
__all__ = ["instance"]

import sys
import threading
import traceback
import uuid
from platform import node as get_hostname
from types import TracebackType
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Type,
)

from neptune.internal.utils.logger import get_logger

_logger = get_logger()

SYS_UNCAUGHT_EXCEPTION_HANDLER_TYPE = Callable[[Type[BaseException], BaseException, Optional[TracebackType]], Any]


class UncaughtExceptionHandler:
    def __init__(self) -> None:
        self._previous_uncaught_exception_handler: Optional[SYS_UNCAUGHT_EXCEPTION_HANDLER_TYPE] = None
        self._handlers: Dict[uuid.UUID, Callable[[List[str]], None]] = dict()
        self._lock = threading.Lock()

    def trigger(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        header_lines = [
            f"An uncaught exception occurred while run was active on worker {get_hostname()}.",
            "Marking run as failed",
            "Traceback:",
        ]

        traceback_lines = header_lines + traceback.format_tb(exc_tb) + str(exc_val).split("\n")
        for _, handler in self._handlers.items():
            handler(traceback_lines)

    def activate(self) -> None:
        with self._lock:
            if self._previous_uncaught_exception_handler is not None:
                return
            self._previous_uncaught_exception_handler = sys.excepthook
            sys.excepthook = self.exception_handler

    def deactivate(self) -> None:
        with self._lock:
            if self._previous_uncaught_exception_handler is None:
                return
            sys.excepthook = self._previous_uncaught_exception_handler
            self._previous_uncaught_exception_handler = None

    def register(self, uid: uuid.UUID, handler: Callable[[List[str]], None]) -> None:
        with self._lock:
            self._handlers[uid] = handler

    def unregister(self, uid: uuid.UUID) -> None:
        with self._lock:
            if uid in self._handlers:
                del self._handlers[uid]

    def exception_handler(self, *args: Any, **kwargs: Any) -> None:
        self.trigger(*args, **kwargs)

        if self._previous_uncaught_exception_handler is not None:
            self._previous_uncaught_exception_handler(*args, **kwargs)


instance = UncaughtExceptionHandler()



#
# Copyright (c) 2023, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
__all__ = ["generate_hash"]

import hashlib


def generate_hash(*descriptors, length: int) -> str:
    hasher = hashlib.sha256()
    for descriptor in descriptors:
        hasher.update(str(descriptor).encode())

    return hasher.hexdigest()[-length:]
